// Unreal Fighting Framework by Kevin Poretti

#include "FFInputBufferComponent.h"

UFFInputBufferComponent::UFFInputBufferComponent()
{
	PrimaryComponentTick.bCanEverTick = false;
}


void UFFInputBufferComponent::AddInput(const FFFInputState& InputState)
{
	InputBuffer.ForcePush(InputState);
}


bool UFFInputBufferComponent::CheckInputSequence(const FFFInputSequence& InputSequence)
{
    int CondIdx = InputSequence.Sequence.Num() - 1;
    int ElapsedFrames = 0;
    for(int InpIdx = InputBuffer.Num() - 1; InpIdx > 1; InpIdx--)
    {
        int32 RequiredButtons = InputSequence.Sequence[CondIdx].RequiredButtons;
        EFFButtonState RequiredButtonState = InputSequence.Sequence[CondIdx].RequiredButtonState;
        int32 PrevInput = InputBuffer[InpIdx - 1].Buttons;
        int32 CurrInput = InputBuffer[InpIdx].Buttons;
        int32 PrevDisable = InputBuffer[InpIdx - 1].DisabledButtons;
        int32 CurrDisable = InputBuffer[InpIdx].DisabledButtons;
        switch (RequiredButtonState)
        {
            // TODO: should it be (PrevInput & RequiredButtons) == RequiredButtons or what we have now?
            case EFFButtonState::BTNS_Pressed:
                if(!(PrevInput & RequiredButtons | PrevDisable) && 
                   CurrInput & RequiredButtons & ~CurrDisable)
                {
                    CondIdx--;
                }
                break;
            case EFFButtonState::BTNS_Released:
                if(PrevInput & RequiredButtons & ~PrevDisable &&
                   !(CurrInput & RequiredButtons | CurrDisable))
                {
                    CondIdx--;
                }
                break;
            case EFFButtonState::BTNS_Down:
                if(CurrInput & RequiredButtons & ~CurrDisable)
                {
                    CondIdx--;
                }
                break;
            // TODO: implement button held condition
            /*
            case EFFButtonState::BTNS_Held:
                break;
            */
        }

        // All conditions were met
        if(CondIdx == -1)
        {
            // disable most recent input
            InputBuffer[InputBuffer.Num() - 1].DisabledButtons |= InputBuffer[InputBuffer.Num() - 1].Buttons;
            InputBuffer[InputBuffer.Num() - 2].DisabledButtons |= InputBuffer[InputBuffer.Num() - 2].Buttons;
            return true;
        }

        ElapsedFrames++;
        if(ElapsedFrames > InputSequence.MaxDuration)
        {
            return false;
        }
    }

	return false;
}
