// Unreal Fighting Framework by Kevin Poretti

#pragma once

// FF includes
#include "Utils/TCircleBuffer.h"

// UE includes
#include "CoreMinimal.h"
#include "Components/ActorComponent.h"

#include "FFInputBufferComponent.generated.h"

UENUM(BlueprintType)
enum class EFFButtonState : uint8
{
    BTNS_Pressed		UMETA(DisplayName="Pressed"),
	BTNS_Released		UMETA(DisplayName="Released"),
    BTNS_Down			UMETA(DisplayName="Down"),    
	//BTNS_Held			UMETA(DisplayName="Held"),
	BTNS_MAX			UMETA(Hidden)
};

/**
 *	Struct representing the state of a player's inputs for one frame
 */
USTRUCT()
struct FFFInputState
{
    GENERATED_BODY()

	FVector2D	MoveAxes;
	FVector2D	LookAxes;

	UPROPERTY(EditAnywhere, Meta = (Bitmask))
	int32		Buttons;
	int32		DisabledButtons;

	FFFInputState()
	    : MoveAxes(FVector2D::ZeroVector)
	    , LookAxes(FVector2D::ZeroVector)
	    , Buttons(0)
	    , DisabledButtons(0)
	{
	}
};

USTRUCT(BlueprintType)
struct FFFInputCondition
{
    GENERATED_BODY()

	// Buttons required for this specific condition to be valid
	UPROPERTY(EditAnywhere)
	int32 RequiredButtons;

	// The button state required for condition to be valid i.e. pressed or released
	UPROPERTY(EditAnywhere)
	EFFButtonState RequiredButtonState;

	FFFInputCondition()
        : RequiredButtons(0)
        , RequiredButtonState(EFFButtonState::BTNS_Pressed)
	{
	}
};

USTRUCT(BlueprintType)
struct FFFInputSequence
{
    GENERATED_BODY()

    UPROPERTY(EditAnywhere)
	TArray<FFFInputCondition> Sequence;

	UPROPERTY(EditAnywhere)
	int32 MaxDuration;

	FFFInputSequence()
	    : MaxDuration(0)
    {
	}
};

UCLASS( ClassGroup=(UnrealFightingFramework), meta=(BlueprintSpawnableComponent) )
class UNREALFIGHTINGFRAMEWORK_API UFFInputBufferComponent : public UActorComponent
{
	GENERATED_BODY()

public:	
	UFFInputBufferComponent();

	void AddInput(const FFFInputState& InputState);

	bool CheckInputSequence(const FFFInputSequence& InputSequence);

protected:
	/** The underlying buffer data structure for holding past input states */
	TCircleBuffer<FFFInputState, 120> InputBuffer;
};
